// -*- C++ -*-

/* 
 * GChemPaint
 * widgetdata.h
 *
 * Copyright (C) 2002-2003
 *
 * Developed by Jean Brfort <jean.brefort@ac-dijon.fr>
 *
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License as 
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 */

#include "config.h"
#include "widgetdata.h"
#include "view.h"
#include "settings.h"
#include "document.h"
#include "operation.h"


bool gcpWidgetData::IsSelected(Object *obj)
{
	std::list<Object*>::iterator i;
	Object* pMol = obj->GetMolecule();
	for (i = SelectedObjects.begin(); i != SelectedObjects.end(); i++)
		if ((*i == obj) || (*i == pMol)) return true;
	return false;
}

void gcpWidgetData::Unselect(Object* obj)
{
	SelectedObjects.remove(obj);
	View->Update(obj);
}

void gcpWidgetData::UnselectAll()
{
	Object* obj;
	while (!SelectedObjects.empty())
	{
		obj = SelectedObjects.front();
		SelectedObjects.pop_front();
		obj->SetSelected(Canvas, SelStateUnselected);
		View->Update(obj);//FIXME: is it really useful
	}
}

void gcpWidgetData::SetSelected(Object* obj)
{
	SelectedObjects.push_front(obj);
	if (obj->GetType() == MoleculeType) UpdateSelected(obj);//FIXME: Should implement SetSelected for Atoms and Bonds
	else obj->SetSelected(Canvas, SelStateSelected);
}

void gcpWidgetData::UpdateSelected(Object* obj)
{
	Object* pObject;
	GnomeCanvasGroup* group;
//	GList* il;
	if ((group = Items[obj])) SetSelected(group);
/*	{
		il = group->item_list;
		while (il)
		{
			if (GNOME_IS_CANVAS_GROUP(il->data)
			else if (!GNOME_IS_CANVAS_RE(il->data)) g_object_set(G_OBJECT(il->data), "fill_color", SelectColor, NULL);
			il = il->next;
		}
	}*/
	else
		Items.erase(obj);
	std::map<std::string, Object*>::iterator i;
	pObject = obj->GetFirstChild(i);
	while (pObject)
	{
		UpdateSelected(pObject);
		pObject = obj->GetNextChild(i);
	}
}

void gcpWidgetData::SetSelected(GnomeCanvasGroup* group)
{
	GList* il = group->item_list;
	while (il)
	{
		if (GNOME_IS_CANVAS_GROUP(il->data)) SetSelected((GnomeCanvasGroup*)il->data);
		else if (GNOME_IS_CANVAS_BPATH(il->data)) g_object_set(G_OBJECT(il->data), "outline_color", SelectColor, NULL);
else if (!GNOME_IS_CANVAS_RE(il->data)) g_object_set(G_OBJECT(il->data), "fill_color", SelectColor, NULL);
		else if (GNOME_IS_CANVAS_ELLIPSE(il->data)) g_object_set(G_OBJECT(il->data), "outline_color", SelectColor, NULL);
		il = il->next;
	}
}

void gcpWidgetData::MoveSelectedItems(double dx, double dy)
{
	std::list<Object*>::iterator i;
	for (i = SelectedObjects.begin(); i != SelectedObjects.end(); i++) MoveItems(*i, dx, dy);
}

void gcpWidgetData::MoveItems(Object* obj, double dx, double dy)
{
	Object* pObject;
	GnomeCanvasGroup* group;
	if ((group = Items[obj])) gnome_canvas_item_move((GnomeCanvasItem*)group, dx, dy);
	else
		Items.erase(obj);
	std::map<std::string, Object*>::iterator i;
	pObject = obj->GetFirstChild(i);
	while (pObject)
	{
		MoveItems(pObject, dx, dy);
		pObject = obj->GetNextChild(i);
	}
}

void gcpWidgetData::MoveSelection(double dx, double dy)
{
	std::list<Object*>::iterator i;
	gcpDocument* pDoc = View->GetDoc();
	gcpOperation* pOp = pDoc-> GetNewModifyOperation();
	for (i = SelectedObjects.begin(); i != SelectedObjects.end(); i++)
	{
		pOp->AddObject(*i,0);
		(*i)->Move(dx / ZoomFactor, dy / ZoomFactor);
		View->Update(*i);
		pOp->AddObject(*i,1);
	}
	pDoc->FinishOperation();
}
